#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <norm.h>

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <net/ethernet.h>
#ifdef LINUX
#define __FAVOR_BSD
#endif
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>

#ifdef FREEBSD
#include <netinet/ip_var.h>
#include <netinet/udp_var.h>
#endif
#include <sys/time.h>
#include <errno.h>
#include <netdb.h>
#include <pcap.h>


#define FDDI_HDR_LEN    13
#define SNAP_LEN        8

static char             errbuf[PCAP_ERRBUF_SIZE];
static pcap_t         *pcap;
static char           *input;
static char           *output;
static pcap_dumper_t  *dumper;   
static u_int           maxlen = 0;
static int             processed = 0;
static int             changed = 0;
static int             increase = 0;

static void
handler(u_char *user, const struct pcap_pkthdr* pkthdr, const u_char * packet)
{
  struct  pcap_pkthdr  newpkthdr;
  u_char              *newpacket;

  if (pkthdr->caplen < pkthdr->len)
    {
      newpacket = calloc(1, pkthdr->len);
      memcpy(newpacket, packet, pkthdr->caplen);
      memcpy(&newpkthdr, pkthdr, sizeof(struct pcap_pkthdr));
      newpkthdr.caplen = newpkthdr.len;

      if (newpkthdr.len > maxlen)
	maxlen = newpkthdr.len;

      pcap_dump((u_char*)dumper, &newpkthdr, newpacket);
      changed++;
      increase += (pkthdr->len - pkthdr->caplen);
      if (newpacket)
	free(newpacket);
    }
  else
    {
      pcap_dump((u_char*)dumper, pkthdr, packet);
    }
  
  processed++;

  return;
  user = NULL;
}


static void
adjust_header(void)
{
  struct pcap_file_header pfh;
  FILE   *f;


  if ( (f = fopen(output, "r+")))
    {
      fread(&pfh, sizeof(struct pcap_file_header), 1, f);
      pfh.snaplen = maxlen;
      fseek(f, 0, SEEK_SET);
      fwrite(&pfh, sizeof(struct pcap_file_header), 1, f);
      fclose(f);
    }
}

int
main(int argc, char **argv)
{
  int     num_packets;

  if (argc < 4)
    {
      printf("USAGE: %s <num packets> <input trace> <output trace>\n", argv[0]);
      exit(1);
    }

  num_packets = atoi(argv[1]);
  input  = argv[2];
  output = argv[3];

  pcap = pcap_open_offline(input, errbuf);
  dumper = pcap_dump_open(pcap, output);
  pcap_loop(pcap, num_packets, handler, NULL);
  pcap_close(pcap);
  pcap_dump_close(dumper);

  adjust_header();

  printf("%i packets processed, %i inflated, total length increase %i bytes.\n",
	 processed, changed, increase);

  return (0);
}
