#ifndef __norm_ip_fragment_h
#define __norm_ip_fragment_h

#include <norm_packet.h>
#include <norm_list.h>

/* What's the maximum number of fragments per packet it's reasonable to
   expect?
   64K / 536bytes ~=122
   If this is exceeded, it's probably a stateholding attack.
   */
#define NORM_MAX_FRAGS_PER_PACKET 300

/* What's the maximum number of fragmented packets it's reasonable to expect?
   We don't know, but can restrict memory usage.
   Allow approx 4MBytes for reassembly.  4MB/64K = 64
   If this is exceeded, perform triage
   */
#define NORM_MAX_FRAGGED_PACKETS 100000

typedef struct norm_fragment     NormFragment;
typedef struct norm_f_packet     NormFPacket;


struct norm_fragment
{
  char               *data;      /* after headers and options have been removed */
  u_int               len;
  u_int               offset;
  u_int               mf;
};

struct norm_f_packet
{
  struct in_addr      src;
  struct in_addr      dst;
  u_int               id;
  u_char             *header;    /* IP header and options from first fragment */
  int                 headerlen; /* length in bytes of header and options     */
  struct timeval      ts;        /* time most recent fragment arrived         */
  int                 fragsize;
  int                 num_frags;
  NormList           *fragments;
};

void norm_ip_fragment_init(void);
int  norm_ip_fragment_handler(NormPacket *p, Interface *nif);

#endif
