#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <unistd.h>
#include <errno.h>

#include <norm.h>
#include <norm_debug.h>
#include <norm_stats.h>

static int       num_packets    = 0;
static int       num_ip_norms   = 0;
static int       num_tcp_norms  = 0;
static int       num_udp_norms  = 0;
static int       num_icmp_norms = 0;
static clock_t   starttime_wall;
static clock_t   starttime_user;
static float     clockticks;

void
norm_stats_init(void)
{
  long clktcks;

  if ( (clktcks = sysconf(_SC_CLK_TCK)) < 0)
    {
      fprintf(stderr, "sysconf error -- exiting.\n");
      exit(errno);
    }

  clockticks = (float)clktcks;
}


int   
norm_stats_increment_num_packets(void)
{
  return (++num_packets);
}


int     
norm_stats_increment_ip_norms(void)
{
  return (++num_ip_norms);
}


int     
norm_stats_increment_tcp_norms(void)
{
  return (++num_tcp_norms);
}


int     
norm_stats_increment_udp_norms(void)
{
  return (++num_udp_norms);
}


int     
norm_stats_increment_icmp_norms(void)
{
  return (++num_icmp_norms);
}


int     
norm_stats_get_ip_norms(void)
{
  return (num_ip_norms);
}


int     
norm_stats_get_tcp_norms(void)
{
  return (num_tcp_norms);
}


int     
norm_stats_get_udp_norms(void)
{
  return (num_udp_norms);
}


int     
norm_stats_get_icmp_norms(void)
{
  return (num_icmp_norms);
}


int     
norm_stats_get_norms(void)
{
  return (num_ip_norms + num_tcp_norms + num_udp_norms + num_icmp_norms);
}


int     
norm_stats_get_num_packets(void)
{
  return (num_packets);
}


void    
norm_stats_start_clock(void)
{
  struct tms t;

  starttime_wall = times(&t);
  starttime_user = t.tms_utime;
}


float   
norm_stats_get_usertime(void)
{
  struct tms t;

  times(&t);
  return ((t.tms_utime - starttime_user) / clockticks);
}


float   
norm_stats_get_walltime(void)
{
  struct tms t;

  return ((times(&t) - starttime_wall) / clockticks);
}


float   
norm_stats_get_pps(void)
{
  return (num_packets / norm_stats_get_usertime());
}


float   
norm_stats_get_pps_calculated(int packets)
{
  return (packets / norm_stats_get_usertime());
}
