#ifndef __norm_tcp_h
#define __norm_tcp_h

#include <norm.h>
#include <netinet/tcp.h>

#ifndef TCPOPT_CC
#define TCPOPT_CC     11
#define TCPOPT_CCNEW  12
#define TCPOPT_CCECHO 13
#endif

#define MAX_WINDOW_ALLOWED 65536


int   norm_tcp_handler(NormPacket *p, Interface *nif);

/* make_reset_reliable is called when a RST is received from the external
 * peer in a connection.  We don't want to tear down state when we get
 * an external RST because it's not necessarily going to be accepted by
 * the internal host, so we allow the RST through, and then afterwards
 * send an ACK for the same connection.  If the RST was successful, the
 * internal peer will generate a RST, which we will trust and which 
 * will cause the TCB to be torn down.
 */
int   norm_tcp_make_reset_reliable(struct ip *ih, struct tcphdr *th, int len, Interface *nif);

/* remove_tcp_option is called to remove the option starting at optp */
int   norm_tcp_remove_option(struct tcphdr *th, unsigned char *optp);

/* fix_options is called when we removed more that four bytes of 
   TCP options. It removes the unnecessary no-op options that we added by
   remove_tcp_option */
int   norm_tcp_fix_options(struct tcphdr *th, int tlen, int remove);

int   norm_tcp_checksum(const struct ip *ip, const struct tcphdr *tp, int len);


#endif
