#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <errno.h>
#include <netdb.h>
#ifdef LINUX
#define __FAVOR_BSD
#endif
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

#ifdef FREEBSD
#include <netinet/ip_var.h>
#include <netinet/udp_var.h>
#endif

#include <norm_ip.h>
#include <norm_udp.h>
#include <norm_debug.h>
#include <norm_options.h>
#include <norm_stats.h>

#ifdef LINUX

struct ipovly
{
  u_int8_t         ih_pr;
  u_int16_t        ih_len;
  struct in_addr   ih_src;
  struct in_addr   ih_dst;
};

#endif

int
norm_udp_checksum_valid(struct udphdr *udphdr, unsigned int preadd)
{
  unsigned short origsum;
  unsigned short newsum;

  D_ENTER;
  
  origsum = udphdr->uh_sum;
  udphdr->uh_sum = 0;

  newsum  = norm_ip_in_cksum((u_short *) udphdr, ntohs(udphdr->uh_ulen), preadd);

  if (newsum == origsum)
    {
      udphdr->uh_sum = origsum;
      D_RETURN_(1);
    }

  udphdr->uh_sum = origsum;
  D_RETURN_(0);
}


int   
norm_udp_handler(NormPacket *p, Interface *nif)
{
  struct ip      *iphdr;
  struct udphdr  *udphdr;
  /* struct servent *srvnt = NULL; */

  D_ENTER;

  iphdr  = (struct ip*)p->network_data;
  udphdr = (struct udphdr*)p->transport_data;

  print_udp(INFO_LOGLEVEL, ("%i", ntohs(udphdr->uh_sport)));

  /*
    srvnt = getservbyport(udphdr->uh_sport, "udp");
    if (srvnt)
    {
    print_udp_(INFO_LOGLEVEL, ("(%s)", srvnt->s_name));
    }
  */

  print_udp_(INFO_LOGLEVEL, ("->%i", ntohs(udphdr->uh_dport)));

  /*
    srvnt = getservbyport(udphdr->uh_dport, "udp");
    if (srvnt)
    {
    print_udp_(INFO_LOGLEVEL, ("(%s)", srvnt->s_name));
    }
  */

  print_udp_(INFO_LOGLEVEL, (" length:%i\n", ntohs(udphdr->uh_ulen)));

  if (options.udp_verify_length)
    {
      if (ntohs(udphdr->uh_ulen) != (ntohs(iphdr->ip_len) - iphdr->ip_hl*4))
	{
	  norm_stats_increment_udp_norms();
	  print_udp(NORM_LOGLEVEL, ("Header's length field (%u) doesn't match actual length (%u) -- dropping.\n",
		    ntohs(udphdr->uh_ulen), (ntohs(iphdr->ip_len) - iphdr->ip_hl*4)));
	  D_RETURN_(-1);
	}
    }

  /* Checksum is zero if sender didn't calculate it */
  if (options.udp_verify_checksum  && udphdr->uh_sum != 0)
    {
      unsigned int i, preadd = 0;
      struct ipovly ipovly;
      register u_short *w = (u_short*)&ipovly;

      bzero(&ipovly, sizeof(struct ipovly));
      ipovly.ih_pr  = iphdr->ip_p;
      ipovly.ih_len = udphdr->uh_ulen;
      ipovly.ih_src = iphdr->ip_src;
      ipovly.ih_dst = iphdr->ip_dst;

      for (i = 0; i < sizeof(struct ipovly)/sizeof(u_short) ; i++)
	preadd += *w++;

      if (!norm_udp_checksum_valid(udphdr, preadd))
	{
	  norm_stats_increment_udp_norms();
	  print_udp(NORM_LOGLEVEL, ("Packet has invalid UDP checksum -- dropping.\n"));
	  D_RETURN_(-1);
	}
      print_udp(NOISE_LOGLEVEL, ("Checksum verified.\n"));
    }

  D_RETURN_(0);
  nif = NULL;
}
